package cn.com.poc.knowledge.rest.impl;

import cn.com.poc.common.utils.BlContext;
import cn.com.poc.knowledge.aggregate.KnowledgeService;
import cn.com.poc.knowledge.constant.KnowledgeConstant;
import cn.com.poc.knowledge.convert.BizKnowledgeInfoConvert;
import cn.com.poc.knowledge.dto.BizKnowledgeDocumentDto;
import cn.com.poc.knowledge.dto.BizKnowledgeInfoDto;
import cn.com.poc.knowledge.dto.TrainKnowledgeDto;
import cn.com.poc.knowledge.dto.UpsertChunkInfoDto;
import cn.com.poc.knowledge.entity.BizKnowledgeDocumentEntity;
import cn.com.poc.knowledge.entity.BizKnowledgeInfoEntity;
import cn.com.poc.knowledge.query.KnowledgeInfosQueryCondition;
import cn.com.poc.knowledge.query.KnowledgeInfosQueryItem;
import cn.com.poc.knowledge.rest.KnowledgeRest;
import cn.com.poc.knowledge.service.BizKnowledgeInfoService;
import cn.com.poc.support.security.oauth.entity.UserBaseEntity;
import cn.com.poc.thirdparty.resource.demand.ai.entity.knowledge.GetKnowledgeChunkInfoResult;
import cn.com.yict.framemax.data.model.PagingInfo;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.lang.Assert;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Component
public class KnowledgeRestImpl implements KnowledgeRest {

    @Resource
    private KnowledgeService knowledgeService;

    @Resource
    private BizKnowledgeInfoService bizKnowledgeInfoService;

    @Override
    public List<BizKnowledgeDocumentDto> uploadDocument(MultipartFile[] documentFiles) throws Exception {
        Assert.notEmpty(documentFiles);
        Assert.isTrue(documentFiles.length <= 5, "Uploaded files can be no larger than five");
        List<BizKnowledgeDocumentDto> result = new ArrayList<>();
        List<BizKnowledgeDocumentEntity> entities = knowledgeService.uploadDocument(documentFiles);
        result = entities.stream().map(entity -> {
            BizKnowledgeDocumentDto dto = new BizKnowledgeDocumentDto();
            BeanUtil.copyProperties(entity, dto);
            return dto;
        }).collect(Collectors.toList());
        return result;
    }

    @Override
    public Boolean delDocument(Integer knowledgeInfoId, Integer kdId) {
        return knowledgeService.delDocument(knowledgeInfoId, kdId);
    }

    @Override
    public Boolean batchDelDocument(Integer knowledgeInfoId, List<Integer> kdId) {
        Assert.notEmpty(kdId);
        boolean result = true;
        for (Integer id : kdId) {
            result = result && knowledgeService.delDocument(knowledgeInfoId, id);
        }
        return result;
    }

    @Override
    public Boolean renameDocument(Integer kdId, String rename) throws Exception {
        return knowledgeService.renameDocument(kdId, rename);
    }

    @Override
    public String enableKnowledge(Integer kdId) throws Exception {
        return knowledgeService.enableKnowledge(kdId);
    }

    @Override
    public Boolean trainKnowledge(TrainKnowledgeDto dto) throws Exception {
        //执行任务
        knowledgeService.trainKnowledge(dto.getKnowledgeInfoId(), dto.getKdIds(), dto.getSegmentationConfig());
        return true;
    }

    @Override
    public List<BizKnowledgeDocumentDto> getList(String search, String trainStatus, PagingInfo pagingInfo) throws Exception {
        List<BizKnowledgeDocumentDto> result = new ArrayList<>();
        List<BizKnowledgeDocumentEntity> entities = knowledgeService.getList(search, trainStatus, pagingInfo);
        if (CollectionUtils.isNotEmpty(entities)) {
            result = entities.stream().map(entity -> {
                BizKnowledgeDocumentDto dto = new BizKnowledgeDocumentDto();
                BeanUtil.copyProperties(entity, dto);
                return dto;
            }).collect(Collectors.toList());
        }
        return result;
    }

    @Override
    public List<BizKnowledgeDocumentDto> getListByKdIds(List<Integer> kdIds) {
        List<BizKnowledgeDocumentDto> res = new ArrayList<>();
        Assert.notEmpty(kdIds);
        List<BizKnowledgeDocumentEntity> entities = knowledgeService.getListByKdIds(kdIds);
        if (CollectionUtils.isNotEmpty(entities)) {
            res = entities.stream().map(entity -> {
                BizKnowledgeDocumentDto dto = new BizKnowledgeDocumentDto();
                BeanUtil.copyProperties(entity, dto);
                return dto;
            }).collect(Collectors.toList());
        }
        return res;
    }

    @Override
    public List<BizKnowledgeInfoDto> getKnowledgeList(String search, String trainStatus, PagingInfo pagingInfo) throws Exception {
        List<BizKnowledgeInfoDto> result = new ArrayList<>();
        UserBaseEntity userBaseEntity = BlContext.getCurrentUserNotException();
        KnowledgeInfosQueryCondition condition = new KnowledgeInfosQueryCondition();
        condition.setMemberId(userBaseEntity.getUserId().toString());
        if (StringUtils.isNoneBlank(search)) {
            condition.setSearch(search);
        }
        if (StringUtils.isNoneBlank(trainStatus)) {
            condition.setTrainStatus(trainStatus);
        }
        List<KnowledgeInfosQueryItem> items = bizKnowledgeInfoService.knowledgeInfos(condition, pagingInfo);
        if (CollectionUtils.isNotEmpty(items)) {
            result = items.stream().map(item -> {
                BizKnowledgeInfoDto dto = new BizKnowledgeInfoDto();
                BeanUtil.copyProperties(item, dto);
                return dto;
            }).collect(Collectors.toList());
        }
        return result;
    }

    @Override
    public BizKnowledgeInfoDto createKnowledge(BizKnowledgeInfoDto dto) throws Exception {
        Assert.notNull(dto.getKnowledgeName(), "知识库名称不能为空");
        UserBaseEntity userBaseEntity = BlContext.getCurrentUserNotException();
        BizKnowledgeInfoEntity bizKnowledgeInfoEntity = new BizKnowledgeInfoEntity();
        bizKnowledgeInfoEntity.setKnowledgeName(dto.getKnowledgeName());
        bizKnowledgeInfoEntity.setMemberId(userBaseEntity.getUserId().toString());
        bizKnowledgeInfoEntity.setTrainStatus(KnowledgeConstant.TrainStatus.UNOPENED);
        bizKnowledgeInfoEntity.setKnowledgeDesc(dto.getDesc());
        BizKnowledgeInfoEntity entity = bizKnowledgeInfoService.save(bizKnowledgeInfoEntity);
        return BizKnowledgeInfoConvert.entityToDto(entity);
    }

    @Override
    public BizKnowledgeInfoDto updateKnowledgeInfo(BizKnowledgeInfoDto dto) throws Exception {
        Assert.notNull(dto.getId());
        Assert.notBlank(dto.getKnowledgeName(), "知识库名称不能为空");
        BizKnowledgeInfoEntity bizKnowledgeInfoEntity = new BizKnowledgeInfoEntity();
        bizKnowledgeInfoEntity.setId(dto.getId());
        bizKnowledgeInfoEntity.setKnowledgeName(dto.getKnowledgeName());
        bizKnowledgeInfoEntity.setKnowledgeDesc(dto.getDesc());
        BizKnowledgeInfoEntity entity = bizKnowledgeInfoService.update(bizKnowledgeInfoEntity);
        return BizKnowledgeInfoConvert.entityToDto(entity);
    }

    @Override
    public void openKnowledgeChunk(UpsertChunkInfoDto dto) {
        Assert.notNull(dto.getIsOpen());
        Assert.notNull(dto.getChunkRelationId());
        Assert.notNull(dto.getKdId());
        knowledgeService.openKnowledgeChunk(dto.getKdId(), dto.getChunkRelationId(), dto.getIsOpen());
    }

    @Override
    public void deleteKnowledgeChunk(UpsertChunkInfoDto dto) {
        Assert.notNull(dto.getKdId());
        Assert.notNull(dto.getChunkRelationId());
        knowledgeService.deleteKnowledgeChunk(dto.getKdId(), dto.getChunkRelationId());
    }

    @Override
    public void updateKnowledgeChunkDoc(UpsertChunkInfoDto dto) {
        Assert.notNull(dto.getChunkContent());
        Assert.notNull(dto.getChunkRelationId());
        Assert.notNull(dto.getKdId());
        knowledgeService.updateKnowledgeChunkDoc(dto.getKdId(), dto.getChunkRelationId(), dto.getChunkContent());
    }

    @Override
    public void addKnowledgeChunk(UpsertChunkInfoDto dto) {
        Assert.notNull(dto.getChunkContent());
        Assert.notNull(dto.getKdId());
        Assert.notNull(dto.getChunkSort());
        knowledgeService.addKnowledgeChunk(dto.getKdId(), dto.getChunkContent(), dto.getChunkSort());
    }

    @Override
    public GetKnowledgeChunkInfoResult getChunks(List<Integer> kdIds, PagingInfo pagingInfo) {
        Assert.notEmpty(kdIds);
        return knowledgeService.getChunkInfo(kdIds, pagingInfo);
    }
}
